<?php

/**
 * @file
 *   The Menu Wizard administration pages.
 */

/**
 * Save an overridden menu item.
 *
 * @param $item array
 *   The menu item to save.
 *
 * @return mixed
 *   The return value of drupal_write_record().
 */
function mew_save(array $item) {
  mew_delete($item['path']);
  foreach (array('title', 'page', 'access') as $callback) {
    $item[$callback . '_arguments'] = serialize($item[$callback . '_arguments']);
  }

  return drupal_write_record('mew', $item);
}

/**
 * Delete an overwritten menu item.
 *
 * @param $path string
 *   The path of the menu item to delete.
 */
function mew_delete($path) {
  db_query("DELETE FROM {mew} WHERE path = '%s'", $path);
}

/**
 * List all available menu links.
 */
function mew_form_list(&$form_state) {
  drupal_add_css(drupal_get_path('module', 'mew') . '/css/mew.css');
  drupal_add_js(drupal_get_path('module', 'mew') . '/js/mew.js');

  $form['keywords'] = array(
    '#type' => 'textfield',
    '#title' => t('Filter menu links on their URL path or title'),
    '#default_value' => isset($form_state['values']) ? $form_state['values']['keywords'] : NULL,
  );
  $form['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Filter'),
    '#attributes' => array(
      'class' => 'js-hide',
    ),
  );
  $keywords = array();
  if (isset($form_state['values']['keywords']) && strlen($form_state['values']['keywords'])) {
    $keywords = explode(' ', $form_state['values']['keywords']);
  }
  $form['list'] = array(
    '#value' => mew_list($keywords),
    '#prefix' => '<div id ="mew-list-wrapper">',
    '#suffix' => '</div>',
  );

  return $form;
}

/**
 * Form submit handler for mew_form_list().
 */
function mew_form_list_submit(array $form, array &$form_state) {
  $form_state['rebuild'] = TRUE;
}

/**
 * List all available menu links.
 */
function mew_list(array $keywords = array()) {
  $items = mew_menu_items();
  $overwritten_paths= array();
  $result = db_query("SELECT path FROM {mew}");
  while ($path = db_result($result)) {
    $overwritten_paths[] = $path;
  }


  // Sort the items.
  ksort($items);
  if (isset($_GET['order']) && $_GET['order'] == t('Link title')) {
    uasort($items, 'mew_sort_title');
  }
  elseif (isset($_GET['order']) && $_GET['order'] == t('Type')) {
    uasort($items, 'mew_sort_type');
  }
  if (isset($_GET['sort']) && $_GET['sort'] == 'desc') {
    $items = array_reverse($items);
  }

  $item_types = mew_item_types();
  $rows = array();
  foreach ($items as $path => $item) {
    if ($keywords) {
      $strings = array($path, $item['title']);
      $matches = array(TRUE, TRUE);
      foreach ($strings as $i => $string) {
        foreach ($keywords as $keyword) {
          if (stripos($string, $keyword) === FALSE) {
            $matches[$i] = FALSE;
          }
        }
      }
      // Not all keywords are present, so skip this menu link.
      if (!$matches[0] && !$matches[1]) {
        continue;
      }
    }

    $reset = $class = NULL;
    if (in_array($path, $overwritten_paths)) {
      $class = 'mew-overwritten';
      $reset = l(t('reset'), 'admin/build/mew/reset/' . $path);
    }
    $rows[] = array(
      'data' => array($path, $item['title'], $item_types[$item['type']], l(t('edit'), 'admin/build/mew/edit/' . $path), $reset),
      'class' => $class,
    );
  }

  $header = array(
    array(
      'data' => t('URL path'),
      'field' => t('path'),
    ),
    array(
      'data' => t('Title'),
      'field' => t('title'),
    ),
    array(
      'data' => t('Type'),
      'field' => t('type'),
    ),
    array(
      'data' => t('Operations'),
      'colspan' => 2,
    ),
  );

  return theme('table', $header, $rows, array('id' => 'mew-list'));
}

/**
 * Sort menu items by title. usort() callback.
 */
function mew_sort_title(array $item_a, array $item_b) {
  return strcmp($item_a['title'], $item_b['title']);
}

/**
 * Sort menu items by type. usort() callback.
 */
function mew_sort_type(array $item_a, array $item_b) {
  static $item_types = NULL;

  if (!$item_types) {
    $item_types = mew_item_types();
  }

  return strcmp($item_types[$item_a['type']], $item_types[$item_b['type']]);
}

/**
 * Return a list of menu item types.
 *
 * @return array
 *   Keys are menu constants and values are human-readable type names.
 */
function mew_item_types() {
  return array(
    MENU_NORMAL_ITEM => t('Normal'),
    MENU_LOCAL_TASK => t('Tab (local task)'),
    MENU_DEFAULT_LOCAL_TASK => t('Default tab (local task)'),
    MENU_CALLBACK => t('Hidden (callback)'),
    MENU_SUGGESTED_ITEM => t('Suggested'),
  );
}

/**
 * Menu link edit form.
 */
function mew_form_edit(&$form_state) {
  // The first 21 characters are "admin/build/mew/edit/", which is the menu
  // item for this form.
  $path = substr($_GET['q'], 21);
  if (!($item = mew_menu_item($path))) {
    drupal_not_found();
    exit;
  }

  drupal_set_title(t('Edit %url_path', array('%url_path' => $path)));

  $form['#redirect'] = 'admin/build/mew';
  $form['path'] = array(
    '#type' => 'value',
    '#value' => $path,
  );
  $form['type'] = array(
    '#type' => 'select',
    '#title' => t('Link type'),
    '#options' => mew_item_types(),
    '#default_value' => $item['type'],
  );
  $form['file'] = array(
    '#type' => 'textfield',
    '#title' => t('Include file'),
    '#default_value' => $item['file'],
  );
  if (user_access('administer site configuration')) {
    $message = t('Entering incorrect function names or arguments may cause your website to malfunction. In case of trouble, <a href="@uninstall">uninstall Menu Wizard</a>.', array('@uninstall' => url('admin/build/modules', array('fragment' => 'edit-status-mew'))));
  }
  else {
    $message = t('Entering incorrect function names or arguments may cause your website to malfunction. In case of trouble, uninstall Menu Wizard.');
  }
  $callback_warning = '<div class="messages warning">' . $message . '</div>';
  $callbacks = array(
    'title' => array(
      '#title' => t('Title callback'),
      '#callback_description' => t('Defaults to !t.', array('!t' => l('t()', 'http://api.drupal.org/api/function/t/7'))),
      '#arguments_description' => t("One argument per line. Integers will match the corresponding URL path component. The menu item's title will automatically be the first argument."),
    ),
    'page' => array(
      '#title' => t('Page callback'),
      '#callback_description' => NULL,
      '#arguments_description' => t('One argument per line. Integers will match the corresponding URL path component.'),
    ),
    'access' => array(
      '#title' => t('Access callback'),
      '#callback_description' => t('Enter <code>TRUE</code> to allow access to everyone.'),
      '#arguments_description' => t('One argument per line. Integers will match the corresponding URL path component.'),
    ),
  );
  foreach ($callbacks as $callback => $callback_info) {
    $form[$callback] = array(
      '#type' => 'fieldset',
      '#title' => $callback_info['#title'],
      '#description' => $callback_warning,
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $form[$callback][$callback . '_callback'] = array(
      '#type' => 'textfield',
      '#title' => t('Callback'),
      '#description' => $callback_info['#callback_description'],
      '#default_value' => $item[$callback . '_callback'],
    );
    $disabled = FALSE;
    foreach ($item[$callback . '_arguments'] as $i => $argument) {
      if (is_object($argument) || is_array($argument)) {
        $item[$callback . '_arguments'][$i] = '#' . gettype($argument) . '#';
        $disabled = TRUE;
      }
    }
    $form[$callback][$callback . '_arguments'] = array(
      '#type' => 'textarea',
      '#title' => t('Arguments'),
      '#description' => $callback_info['#arguments_description'],
      '#default_value' => implode("\n", $item[$callback . '_arguments']),
      '#disabled' => $disabled,
    );
  }
  $form['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  if (db_result(db_query("SELECT COUNT(*) FROM {mew} WHERE path = '%s'", $path))) {
    $form['buttons']['reset'] = array(
      '#type' => 'markup',
      '#value' => l(t('Reset'), 'admin/build/mew/reset/' . $path),
    );
  }

  return $form;
}

/**
 * Submit callback for mew_form_edit().
 */
function mew_form_edit_validate(array $form, array &$form_state) {
  $values = $form_state['values'];

  // Confirm that the include file exists.
  if (!empty($values['file']) && !file_exists($values['file'])) {
    form_set_error('file', t('The include file does not exist.'));
  }
  // Confirm that callbacks exist.
  else {
    include($values['file']);
    if (!empty($values['title_callback'])) {
      if (!function_exists($values['title_callback'])) {
        form_set_error('title_callback', t('The title callback cannot be found.'));
      }
      $reflection = new ReflectionFunction($values['title_callback']);
      $count = $reflection->getNumberOfRequiredParameters() - 1;
      if (count(preg_split('#\v#', $values['title_arguments'], NULL, PREG_SPLIT_NO_EMPTY)) < $count) {
        form_set_error('title_arguments', t('The title callback requires at least !count arguments.', array('!count' => $count)));
      }
    }
    if (!empty($values['page_callback'])) {
      if (!function_exists($values['page_callback'])) {
        form_set_error('page_callback', t('The page callback cannot be found.'));
      }
      $reflection = new ReflectionFunction($values['page_callback']);
      $count = $reflection->getNumberOfRequiredParameters();
      if (count(preg_split('#\v#', $values['page_arguments'], NULL, PREG_SPLIT_NO_EMPTY)) < $count) {
        form_set_error('page_arguments', t('The page callback requires at least !count arguments.', array('!count' => $count)));
      }
    }
    if (!empty($values['access_callback'])) {
      if (!function_exists($values['access_callback'])) {
        form_set_error('access_callback', t('The access callback cannot be found.'));
      }
      $reflection = new ReflectionFunction($values['access_callback']);
      $count = $reflection->getNumberOfRequiredParameters();
      if (count(preg_split('#\v#', $values['access_arguments'], NULL, PREG_SPLIT_NO_EMPTY)) < $count) {
        form_set_error('access_arguments', t('The access callback requires at least !count arguments.', array('!count' => $count)));
      }
    }
  }
}

/**
 * Submit callback for mew_form_edit().
 */
function mew_form_edit_submit(array $form, array &$form_state) {
  $item = $form_state['values'];
  // The title callback defaults to t().
  if (empty($item['title_callback'])) {
    $item['title_callback'] = 't';
  }
  // The access callback can be TRUE to allow access to everyone.
  if ($item['access_callback'] == 'TRUE') {
    $item['access_callback'] = TRUE;
  }
  // Convert callback arguments to arrays.
  foreach (array('title', 'page', 'access') as $callback) {
    $item[$callback . '_arguments'] = preg_split('#\v#', $item[$callback . '_arguments'], NULL, PREG_SPLIT_NO_EMPTY);
    // Convert numbers to integers, so they match URL path components.
    foreach ($item[$callback . '_arguments'] as &$argument) {
      if (!preg_match('#\D#', $argument)) {
        $argument = (int) $argument;
      }
    }
  }
  mew_save($item);
  mew_menu_rebuild();
  drupal_set_message(t('The menu item for %path has been overwritten.', array('%path' => $item['path'])));
}

/**
 * Form: reset a menu link.
 *
 * @return array
 *   A Drupal form.
 */
function mew_form_reset(array &$form_state) {
  // The first 22 characters are "admin/build/mew/reset/", which is the menu
  // item for this form.
  $path = substr($_GET['q'], 22);

  if (!mew_menu_item($path)) {
    drupal_not_found();
    exit;
  }

  $form['#redirect'] = 'admin/build/mew';
  $form['path'] = array(
    '#type' => 'value',
    '#value' => $path,
  );

  return confirm_form($form,
    t('Are you sure you want to reset %title?', array('%title' => $path)),
    isset($_GET['destination']) ? $_GET['destination'] : 'admin/build/mew',
    t('This action cannot be undone.'),
    t('Reset'),
    t('Cancel')
  );
}

/**
 * Submit callback for mew_form_reset().
 */
function mew_form_reset_submit(array $form, array &$form_state) {
  mew_delete($form_state['values']['path']);
  mew_menu_rebuild(); 
  drupal_set_message(t('The menu item for %path has been reset.', array('%path' => $form_state['values']['path'])));
}

/**
 * Rebuild the menu after changing menu items.
 */
function mew_menu_rebuild() {
  // Menu items can appear twice after overwriting them. The following
  // workaround prevents this: disable all modules, rebuild the menu, enable
  // all modules and rebuild the menu again. This doesn't work if all modules
  // are disabled, because then there are no menu items, so we do this
  // separately for Menu Wizard.
  // Get all currently set messages so we can set them again later to hide
  // messages that are set when enabling modules.
  $messages = drupal_get_messages();
  $modules = module_list();
  unset($modules['mew']);
  module_disable($modules);
  menu_rebuild();
  module_enable($modules);
  menu_rebuild();
  module_disable(array('mew'));
  menu_rebuild();
  module_enable(array('mew'));
  menu_rebuild();
  // Clear messages that were set in the meantime and set the old ones again.
  drupal_get_messages();
  foreach ($messages as $type => $type_messages) {
    foreach ($type_messages as $message) {
      drupal_set_message($message, $type);
    }
  }
}